@@ -64,9 +64,11 @@ int lnx_queue_msg(struct fi_peer_rx_entry *entry)
 
 void lnx_free_entry(struct fi_peer_rx_entry *entry)
 {
-	struct lnx_rx_entry *rx_entry = (struct lnx_rx_entry *) entry;
+	struct lnx_rx_entry *rx_entry;
 	ofi_spin_t *bplock;
 
+	rx_entry = container_of(entry, struct lnx_rx_entry, rx_entry);
+
 	if (rx_entry->rx_global)
 		bplock = &global_bplock;
 	else
@@ -175,16 +177,16 @@ lnx_remove_first_match(struct lnx_queue *q, struct lnx_match_attr *match)
 static inline void
 lnx_insert_rx_entry(struct lnx_queue *q, struct lnx_rx_entry *entry)
 {
-	dlist_insert_tail((struct dlist_entry *)(&entry->rx_entry),
-			  &q->lq_queue);
+	dlist_insert_tail(&entry->entry, &q->lq_queue);
 	lnx_update_queue_stats(q, false);
 }
 
 int lnx_queue_tag(struct fi_peer_rx_entry *entry)
 {
-	struct lnx_rx_entry *rx_entry = (struct lnx_rx_entry *)entry;
+	struct lnx_rx_entry *rx_entry;
 	struct lnx_peer_srq *lnx_srq = (struct lnx_peer_srq*)entry->owner_context;
 
+	rx_entry = container_of(entry, struct lnx_rx_entry, rx_entry);
 	FI_DBG(&lnx_prov, FI_LOG_CORE,
 		"addr = %lx tag = %lx ignore = 0 found\n",
 		entry->addr, entry->tag);
@@ -556,9 +558,6 @@ ssize_t lnx_tsendv(struct fid_ep *ep, const struct iovec *iov, void **desc,
 	struct lnx_core_ep *cep;
 	fi_addr_t core_addr;
 
-	if (count > 1)
-		return -FI_EINVAL;
-
 	lep = container_of(ep, struct lnx_ep, le_ep.ep_fid.fid);
 	if (!lep)
 		return -FI_ENOSYS;
@@ -595,9 +594,6 @@ ssize_t lnx_tsendmsg(struct fid_ep *ep, const struct fi_msg_tagged *msg,
 	struct lnx_core_ep *cep;
 	struct fi_msg_tagged core_msg;
 
-	if (msg->iov_count > 1)
-		return -FI_EINVAL;
-
 	memcpy(&core_msg, msg, sizeof(*msg));
 
 	lep = container_of(ep, struct lnx_ep, le_ep.ep_fid.fid);
