@@ -232,11 +232,11 @@ unsigned fi_opx_hfi1_error_inject(struct fi_opx_ep *opx_ep, uint64_t *p_rhf_seq,
 	 * Error injection .. purposefully drop packet
 	 */
 	if (OFI_UNLIKELY(FI_OPX_RELIABILITY_RX_DROP_PACKET(opx_ep->reli_service, hdr))) {
-		*p_rhf_seq   = OPX_RHF_SEQ_INCREMENT(rhf_seq, OPX_HFI1_TYPE);
+		*p_rhf_seq   = OPX_RHF_SEQ_INCREMENT(rhf_seq, OPX_SW_HFI1_TYPE);
 		*p_hdrq_head = hdrq_offset + FI_OPX_HFI1_HDRQ_ENTRY_SIZE_DWS;
 
-		if (OPX_RHF_IS_USE_EGR_BUF(rhf, OPX_HFI1_TYPE)) { /* eager */
-			const uint32_t egrbfr_index	 = OPX_RHF_EGR_INDEX(rhf, OPX_HFI1_TYPE);
+		if (OPX_RHF_IS_USE_EGR_BUF(rhf, OPX_SW_HFI1_TYPE)) { /* eager */
+			const uint32_t egrbfr_index	 = OPX_RHF_EGR_INDEX(rhf, OPX_SW_HFI1_TYPE);
 			const uint32_t last_egrbfr_index = *p_last_egrbfr_index;
 			if (OFI_UNLIKELY(last_egrbfr_index != egrbfr_index)) {
 				OPX_HFI1_BAR_UREG_STORE(egrq_head_reg, (const uint64_t) last_egrbfr_index);
@@ -426,7 +426,7 @@ void opx_write_header_to_subctxt(struct opx_software_rx_q *soft_rx_q, const uint
 	const uint64_t hdrq_offset_dws = (rhf_msb >> 12) & 0x01FFu;
 
 	uint32_t *pu32;
-	if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR_9B)) {
+	if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_MIXED_9B)) {
 		assert(hdrq_offset_dws); /* need padding before this header */
 		pu32 = (uint32_t *) rhf_ptr_dest - FI_OPX_HFI1_HDRQ_ENTRY_SIZE_DWS + 2 /* rhf field size in dw */
 		       - 2 /* sizeof(uint64_t) in dw, offset back to align
@@ -613,7 +613,7 @@ unsigned fi_opx_hfi1_poll_once(struct fid_ep *ep, const int lock_required, const
 		const uint64_t hdrq_offset_dws = (rhf_msb >> 12) & 0x01FFu;
 
 		uint32_t *pkt;
-		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR_9B)) {
+		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_MIXED_9B)) {
 			assert(hdrq_offset_dws); /* need padding before this header */
 			pkt = (uint32_t *) rhf_ptr - FI_OPX_HFI1_HDRQ_ENTRY_SIZE_DWS + 2 /* rhf field size in dw */
 			      - 2 /* sizeof(uint64_t) in dw, offset back to align
@@ -634,9 +634,8 @@ unsigned fi_opx_hfi1_poll_once(struct fid_ep *ep, const int lock_required, const
 
 		/* CYR only has 2 bits available in BTH.QP[15:8] for storing the subctxt value, while WFR/JKR uses 3
 		 * bits.*/
-		const uint8_t subctxt_dest = (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR | OPX_HFI1_JKR_9B)) ?
-						     (hdr->bth.subctxt_rx & 0x7) :
-						     ((hdr->bth.subctxt_rx >> 1) & 0x3);
+		const uint8_t subctxt_dest = (!(OPX_IS_EXTENDED_RX(hfi1_type))) ? (hdr->bth.subctxt_rx & 0x7) :
+										  ((hdr->bth.subctxt_rx >> 1) & 0x3);
 
 		uint64_t  hdrq_head_local;
 		uint32_t *p_last_egrbfr_index;
@@ -684,7 +683,7 @@ unsigned fi_opx_hfi1_poll_once(struct fid_ep *ep, const int lock_required, const
 			}
 		}
 
-		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR_9B)) {
+		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_MIXED_9B)) {
 			slid = (opx_lid_t) __be16_to_cpu24((__be16) hdr->lrh_9B.slid);
 			dlid = (opx_lid_t) __be16_to_cpu24((__be16) hdr->lrh_9B.dlid);
 		} else {
@@ -780,7 +779,7 @@ static inline void fi_opx_shm_poll_many(struct fid_ep *ep, const int lock_requir
 
 #ifndef NDEBUG
 		opx_lid_t dlid __attribute__((unused));
-		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR_9B)) {
+		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_MIXED_9B)) {
 			dlid = (opx_lid_t) __be16_to_cpu24((__be16) hdr->lrh_9B.dlid);
 		} else {
 			dlid = (opx_lid_t) __le24_to_cpu((hdr->lrh_16B.dlid20 << 20) | (hdr->lrh_16B.dlid));
@@ -788,7 +787,7 @@ static inline void fi_opx_shm_poll_many(struct fid_ep *ep, const int lock_requir
 		assert(dlid == opx_ep->rx->self.lid);
 #endif
 
-		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR_9B)) {
+		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_MIXED_9B)) {
 			slid = (opx_lid_t) __be16_to_cpu24((__be16) hdr->lrh_9B.slid);
 		} else {
 			slid = (opx_lid_t) __le24_to_cpu(hdr->lrh_16B.slid20 << 20 | hdr->lrh_16B.slid);
@@ -1039,7 +1038,7 @@ unsigned opx_process_soft_rx_q(struct fi_opx_ep *opx_ep, uint8_t subctxt, const
 		const uint64_t hdrq_offset_dws = (rhf_msb >> 12) & 0x01FFu;
 
 		uint32_t *pkt;
-		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR_9B)) {
+		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_MIXED_9B)) {
 			assert(hdrq_offset_dws); /* need padding before this header */
 			pkt = (uint32_t *) rhf_ptr - FI_OPX_HFI1_HDRQ_ENTRY_SIZE_DWS + 2 /* rhf field size in dw */
 			      - 2 /* sizeof(uint64_t) in dw, offset back to align
@@ -1063,7 +1062,7 @@ unsigned opx_process_soft_rx_q(struct fi_opx_ep *opx_ep, uint8_t subctxt, const
 		const union opx_hfi1_packet_hdr *const hdr    = (union opx_hfi1_packet_hdr *) pkt;
 		const uint8_t			       opcode = hdr->bth.opcode;
 
-		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_JKR_9B)) {
+		if (hfi1_type & (OPX_HFI1_WFR | OPX_HFI1_MIXED_9B)) {
 			slid = (opx_lid_t) __be16_to_cpu24((__be16) hdr->lrh_9B.slid);
 			dlid = (opx_lid_t) __be16_to_cpu24((__be16) hdr->lrh_9B.dlid);
 		} else {
@@ -1127,6 +1126,10 @@ void opx_hfi1_poll_hfi(struct fid_ep *ep, const enum ofi_reliability_kind reliab
 	unsigned	      hfi1_poll_count = 0;
 	unsigned	      packets	      = 0;
 
+#if HAVE_HFISVC
+	uint32_t start_rdma_read_count = opx_ep->hfisvc.rdma_read_count;
+#endif
+
 	if (ctx_sharing) {
 		const uint8_t subctxt_self = opx_ep->rx->shd_ctx.subctxt;
 
@@ -1161,6 +1164,19 @@ void opx_hfi1_poll_hfi(struct fid_ep *ep, const enum ofi_reliability_kind reliab
 							ctx_sharing);
 		} while ((packets > 0) && (hfi1_poll_count++ < hfi1_poll_max));
 	}
+
+#if HAVE_HFISVC
+	if (opx_ep->hfisvc.rdma_read_count != start_rdma_read_count) {
+		FI_OPX_DEBUG_COUNTERS_INC(opx_ep->debug_counters.hfisvc.doorbell_ring.poll_many);
+		OPX_HFISVC_DEBUG_LOG(
+			"Rang doorbell because start_rdma_read_count=%u and opx_ep->hfisvc.rdma_read_count=%u\n",
+			start_rdma_read_count, opx_ep->hfisvc.rdma_read_count);
+		int rc __attribute__((unused));
+		rc = (*opx_ep->domain->hfisvc.doorbell)(opx_ep->domain->hfisvc.ctx);
+
+		assert(rc == 0);
+	}
+#endif
 }
 
 __OPX_FORCE_INLINE__
@@ -1213,6 +1229,19 @@ void fi_opx_hfi1_poll_many(struct fid_ep *ep, const int lock_required, const uin
 			// Drain all coalesced pings
 			fi_opx_hfi_rx_reliablity_process_requests(ep, PENDING_RX_RELIABLITY_COUNT_MAX);
 			fi_reliability_service_ping_remote(ep, service);
+
+#if HAVE_HFISVC
+#ifdef OPX_HFISVC_RELIABILITY_DOORBELL
+			if (opx_ep->use_hfisvc && (opx_ep->tx->cq_pending_ptr || opx_ep->rx->cq_pending_ptr)) {
+				FI_OPX_DEBUG_COUNTERS_INC(
+					opx_ep->debug_counters.hfisvc.doorbell_ring.reliability_timer_pop);
+				int rc __attribute__((unused));
+				rc = (*opx_ep->domain->hfisvc.doorbell)(opx_ep->domain->hfisvc.ctx);
+				assert(rc == 0);
+			}
+#endif
+#endif
+
 			// Fetch the timer again as it could have taken us a while to get through reliability
 			compare		   = fi_opx_timer_now(timestamp, timer);
 			service->usec_next = fi_opx_timer_next_event_usec(timer, timestamp, service->usec_max);
