@@ -346,7 +346,7 @@ static int util_queue_msg(struct fi_peer_rx_entry *rx_entry)
 
 	util_entry = container_of(rx_entry, struct util_rx_entry, peer_entry);
 	assert(util_entry->status == RX_ENTRY_UNEXP);
-	if (rx_entry->addr == FI_ADDR_UNSPEC) {
+	if (!srx_ctx->dir_recv || rx_entry->addr == FI_ADDR_UNSPEC) {
 		dlist_insert_tail(&util_entry->d_entry,
 				  &srx_ctx->unspec_unexp_msg_queue);
 	} else {
@@ -371,7 +371,7 @@ static int util_queue_tag(struct fi_peer_rx_entry *rx_entry)
 
 	util_entry = container_of(rx_entry, struct util_rx_entry, peer_entry);
 	assert(util_entry->status == RX_ENTRY_UNEXP);
-	if (rx_entry->addr == FI_ADDR_UNSPEC) {
+	if (!srx_ctx->dir_recv || rx_entry->addr == FI_ADDR_UNSPEC) {
 		dlist_insert_tail(&util_entry->d_entry,
 				  &srx_ctx->unspec_unexp_tag_queue);
 	} else {
@@ -455,7 +455,8 @@ static void util_foreach_unspec(struct fid_peer_srx *srx,
 				     struct util_rx_entry, rx_entry, d_entry,
 				     tmp) {
 		rx_entry->peer_entry.addr = get_addr(&rx_entry->peer_entry);
-		if (rx_entry->peer_entry.addr == FI_ADDR_UNSPEC)
+		if (rx_entry->peer_entry.addr == FI_ADDR_UNSPEC ||
+		    !srx_ctx->dir_recv)
 			continue;
 
 		dlist_remove(&rx_entry->d_entry);
@@ -472,7 +473,8 @@ static void util_foreach_unspec(struct fid_peer_srx *srx,
 				     struct util_rx_entry, rx_entry, d_entry,
 				     tmp) {
 		rx_entry->peer_entry.addr = get_addr(&rx_entry->peer_entry);
-		if (rx_entry->peer_entry.addr == FI_ADDR_UNSPEC)
+		if (rx_entry->peer_entry.addr == FI_ADDR_UNSPEC ||
+		    !srx_ctx->dir_recv)
 			continue;
 
 		dlist_remove(&rx_entry->d_entry);
