@@ -113,6 +113,7 @@ static void ofi_tostr_addr_format(char *buf, size_t len, uint32_t addr_format)
 	switch (addr_format) {
 	CASEENUMSTRN(FI_FORMAT_UNSPEC, len);
 	CASEENUMSTRN(FI_SOCKADDR, len);
+	CASEENUMSTRN(FI_SOCKADDR_IP, len);
 	CASEENUMSTRN(FI_SOCKADDR_IN, len);
 	CASEENUMSTRN(FI_SOCKADDR_IN6, len);
 	CASEENUMSTRN(FI_SOCKADDR_IB, len);
@@ -150,10 +151,8 @@ ofi_tostr_threading(char *buf, size_t len, enum fi_threading threading)
 	switch (threading) {
 	CASEENUMSTRN(FI_THREAD_UNSPEC, len);
 	CASEENUMSTRN(FI_THREAD_SAFE, len);
-	CASEENUMSTRN(FI_THREAD_FID, len);
 	CASEENUMSTRN(FI_THREAD_DOMAIN, len);
 	CASEENUMSTRN(FI_THREAD_COMPLETION, len);
-	CASEENUMSTRN(FI_THREAD_ENDPOINT, len);
 	default:
 		ofi_strncatf(buf, len, "Unknown");
 		break;
@@ -183,19 +182,6 @@ static void ofi_tostr_msgorder(char *buf, size_t len, uint64_t flags)
 	ofi_remove_comma(buf);
 }
 
-static void ofi_tostr_comporder(char *buf, size_t len, uint64_t flags)
-{
-	if ((flags & FI_ORDER_STRICT) == FI_ORDER_NONE) {
-		ofi_strncatf(buf, len, "FI_ORDER_NONE, ");
-	} else if ((flags & FI_ORDER_STRICT) == FI_ORDER_STRICT) {
-		ofi_strncatf(buf, len, "FI_ORDER_STRICT, ");
-	}
-
-	IFFLAGSTRN(flags, FI_ORDER_DATA, len);
-
-	ofi_remove_comma(buf);
-}
-
 static void ofi_tostr_caps(char *buf, size_t len, uint64_t caps)
 {
 	IFFLAGSTRN(caps, FI_MSG, len);
@@ -226,6 +212,8 @@ static void ofi_tostr_caps(char *buf, size_t len, uint64_t caps)
 	IFFLAGSTRN(caps, FI_SOURCE, len);
 	IFFLAGSTRN(caps, FI_NAMED_RX_CTX, len);
 	IFFLAGSTRN(caps, FI_DIRECTED_RECV, len);
+	IFFLAGSTRN(caps, FI_AV_USER_ID, len);
+	IFFLAGSTRN(caps, FI_PEER, len);
 	IFFLAGSTRN(caps, FI_HMEM, len);
 
 	ofi_remove_comma(buf);
@@ -271,6 +259,8 @@ static void ofi_tostr_protocol(char *buf, size_t len, uint32_t protocol)
 	CASEENUMSTRN(FI_PROTO_XNET, len);
 	CASEENUMSTRN(FI_PROTO_SM2, len);
 	CASEENUMSTRN(FI_PROTO_CXI_RNR, len);
+	CASEENUMSTRN(FI_PROTO_LPP, len);
+	CASEENUMSTRN(FI_PROTO_LNX, len);
 	default:
 		ofi_strncatf(buf, len, "Unknown");
 		break;
@@ -283,9 +273,8 @@ static void ofi_tostr_mode(char *buf, size_t len, uint64_t mode)
 	IFFLAGSTRN(mode, FI_MSG_PREFIX, len);
 	IFFLAGSTRN(mode, FI_ASYNC_IOV, len);
 	IFFLAGSTRN(mode, FI_RX_CQ_DATA, len);
-	IFFLAGSTRN(mode, FI_LOCAL_MR, len);
+	IFFLAGSTRN2(mode, OFI_LOCAL_MR, FI_LOCAL_MR, len);
 	IFFLAGSTRN(mode, FI_CONTEXT2, len);
-	IFFLAGSTRN(mode, FI_BUFFERED_RECV, len);
 
 	ofi_remove_comma(buf);
 }
@@ -333,10 +322,6 @@ ofi_tostr_tx_attr(char *buf, size_t len, const struct fi_tx_attr *attr,
 	ofi_tostr_msgorder(buf, len, attr->msg_order);
 	ofi_strncatf(buf, len, " ]\n");
 
-	ofi_strncatf(buf, len, "%s%scomp_order: [ ", prefix, TAB);
-	ofi_tostr_comporder(buf, len, attr->comp_order);
-	ofi_strncatf(buf, len, " ]\n");
-
 	ofi_strncatf(buf, len, "%s%sinject_size: %zu\n", prefix, TAB,
 		     attr->inject_size);
 	ofi_strncatf(buf, len, "%s%ssize: %zu\n", prefix, TAB, attr->size);
@@ -373,12 +358,6 @@ ofi_tostr_rx_attr(char *buf, size_t len, const struct fi_rx_attr *attr,
 	ofi_tostr_msgorder(buf, len, attr->msg_order);
 	ofi_strncatf(buf, len, " ]\n");
 
-	ofi_strncatf(buf, len, "%s%scomp_order: [ ", prefix, TAB);
-	ofi_tostr_comporder(buf, len, attr->comp_order);
-	ofi_strncatf(buf, len, " ]\n");
-
-	ofi_strncatf(buf, len, "%s%stotal_buffered_recv: %zu\n", prefix, TAB,
-		     attr->total_buffered_recv);
 	ofi_strncatf(buf, len, "%s%ssize: %zu\n", prefix, TAB, attr->size);
 	ofi_strncatf(buf, len, "%s%siov_limit: %zu\n", prefix, TAB,
 		     attr->iov_limit);
@@ -457,8 +436,8 @@ static void ofi_tostr_av_type(char *buf, size_t len, enum fi_av_type type)
 
 static void ofi_tostr_mr_mode(char *buf, size_t len, int mr_mode)
 {
-	IFFLAGSTRN(mr_mode, FI_MR_BASIC, len);
-	IFFLAGSTRN(mr_mode, FI_MR_SCALABLE, len);
+	IFFLAGSTRN2(mr_mode, OFI_MR_BASIC, FI_MR_BASIC, len);
+	IFFLAGSTRN2(mr_mode, OFI_MR_SCALABLE, FI_MR_SCALABLE, len);
 	IFFLAGSTRN(mr_mode, FI_MR_LOCAL, len);
 	IFFLAGSTRN(mr_mode, FI_MR_RAW, len);
 	IFFLAGSTRN(mr_mode, FI_MR_VIRT_ADDR, len);
@@ -511,11 +490,8 @@ ofi_tostr_domain_attr(char *buf, size_t len, const struct fi_domain_attr *attr,
 	ofi_tostr_threading(buf, len, attr->threading);
 	ofi_strncatf(buf, len, "\n");
 
-	ofi_strncatf(buf, len, "%s%scontrol_progress: ", prefix,TAB);
-	ofi_tostr_progress(buf, len, attr->control_progress);
-	ofi_strncatf(buf, len, "\n");
-	ofi_strncatf(buf, len, "%s%sdata_progress: ", prefix, TAB);
-	ofi_tostr_progress(buf, len, attr->data_progress);
+	ofi_strncatf(buf, len, "%s%sprogress: ", prefix, TAB);
+	ofi_tostr_progress(buf, len, attr->progress);
 	ofi_strncatf(buf, len, "\n");
 	ofi_strncatf(buf, len, "%s%sresource_mgmt: ", prefix, TAB);
 	ofi_tostr_resource_mgmt(buf, len, attr->resource_mgmt);
